import { getAllChapters, getLatestChapter } from './mangadex/chapters';
import { getManga } from './mangadex/manga';
import { getCover } from './mangadex/cover';
import { lazyKv } from './db/lazyKv';
import { sendWebhook } from './utils/webhook';
import { flatten, uniq } from 'lodash-es';
import { ChapterId, MangaByWebhook } from './types';

const config = new lazyKv('./config.json');
const mangaHistory = new lazyKv('./mangaHistory.json');

void (async () => {
  const checkInterval = await config.get<number>('checkInterval', 900);
  const mangaByWebhook = await config.get<MangaByWebhook>('mangaByWebhook', {});

  const uniqueMangaIds = uniq(flatten(Object.values(mangaByWebhook)));
  const mangaIdsToWebhooks = uniqueMangaIds.reduce((acc, mangaId) => {
    return {
      ...acc,
      [mangaId]: Object.keys(mangaByWebhook).filter((webhookUrl) =>
        mangaByWebhook[webhookUrl].includes(mangaId),
      ),
    };
  }, {} as MangaByWebhook);

  console.log(
    `Config loaded (${uniqueMangaIds.length} Manga, ${Object.keys(mangaByWebhook).length} Webhooks) `,
  );

  const checkForUpdates = async () => {
    console.log('\nChecking for updates...');

    for (const mangaId of uniqueMangaIds) {
      const lastChapterId = await mangaHistory.get<ChapterId>(mangaId);

      const manga = await getManga(mangaId);
      const chapters = await getAllChapters(mangaId);
      const latestChapter = getLatestChapter(chapters);
      const cover = await getCover(manga);

      if (lastChapterId !== latestChapter.id) {
        console.log(
          'Update found for manga:',
          manga.attributes.title.en || `MISSING ENGLISH TITLE  (${mangaId})`,
        );

        const webhooksForManga = mangaIdsToWebhooks[mangaId];

        await Promise.all(
          webhooksForManga.map(
            async (webhookUrl) =>
              await sendWebhook({
                webhookUrl,
                manga,
                latestChapter,
                cover,
              }),
          ),
        );

        await mangaHistory.set(mangaId, latestChapter.id);
      } else {
        console.log(
          'No Updates found for manga:',
          manga.attributes.title.en || `MISSING ENGLISH TITLE (${mangaId})`,
        );
      }
    }
  };

  setInterval(() => void checkForUpdates(), checkInterval * 1000);
  await checkForUpdates();
})();
